/*----------------------------------------------------------------------------*/
/*  vdcontentviewer.cpp                                                       */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2005 by Dao Hai Lam                               */
/*                      HPT Vietnam Corporation - HPT Commerce                */
/*                      139B Pasteur, District 3,                             */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.hptvietnam.com.vn                          */
/*  email             : lamdh at hptvietnam dot com dot vn                    */
/*                                                                            */
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include "vdcontentviewer.h"
#include "vdwidgets.h"
#include "resource.h"
/*----------------------------------------------------------------------------*/
#include "images.h"
/*----------------------------------------------------------------------------*/
#include <qlayout.h>
#include <qtooltip.h>
#include <qtoolbutton.h>
#include <qprocess.h>
#include <stdlib.h>
/*----------------------------------------------------------------------------*/
#define FESTIVAL_CMD	0
#define FESTIVAL_SRV	1
/*----------------------------------------------------------------------------*/
VDFestival::VDFestival(): runMode(-1)
{
	socket = new QSocket();
	process = new QProcess(), 
	connect( process, SIGNAL( readyReadStdout() ), SLOT( ignore() ) );
	connect( process, SIGNAL( readyReadStderr() ), SLOT( ignore() ) );
	connect( socket, SIGNAL( connected() ), SLOT( connected() ) );
	connect( socket, SIGNAL( readyRead() ), SLOT( ignore() ) );
}
/*----------------------------------------------------------------------------*/
VDFestival::~VDFestival()
{
	delete socket;
	delete process;
}
/*----------------------------------------------------------------------------*/
bool VDFestival::running()
{
	return runMode != -1;
}
/*----------------------------------------------------------------------------*/
void VDFestival::open()
{
	runMode = Resource::speechMode;
	if ( runMode == FESTIVAL_CMD ) {
		process->addArgument( Resource::speechInfo );
		if (process->start()) emit ready();
	}
	else {
		QString host = Resource::speechInfo.section( ':', 0, 0 );
		QString port = Resource::speechInfo.section( ':', 1, 1 );
		if (host == QString::null || host == "") host = "127.0.0.1";
		if (port == QString::null || host == "") port = "1314";
		socket->connectToHost( host, port.toInt() );
	}
}
/*----------------------------------------------------------------------------*/
void VDFestival::close()
{
	const static char *quit = "(quit)\r\n";

	if ( runMode == FESTIVAL_CMD ) {
		process->writeToStdin( quit );
		process->clearArguments();
	}
	else {
		socket->writeBlock( quit, strlen(quit) );
		socket->close();
	}
	runMode = -1;

	emit stop();
}
/*----------------------------------------------------------------------------*/
void VDFestival::reset()
{
	if (running()) close();
	if (Resource::speechEnabled) open();
}
/*----------------------------------------------------------------------------*/
void VDFestival::connected()
{
	emit ready();
}
/*----------------------------------------------------------------------------*/
void VDFestival::ignore()
{
	char dump[256];
	if ( runMode == FESTIVAL_CMD ) {
		while ( process->canReadLineStdout() )
			process->readLineStdout();
	}
	else {
		while ( socket->bytesAvailable() ) {
			int n = socket->bytesAvailable();
			if( n > sizeof(dump) ) n = sizeof(dump);
			socket->readBlock(dump, n);
		}
	}
}
/*----------------------------------------------------------------------------*/
void VDFestival::say( const QString &word )
{
	QString cmd;
	cmd.sprintf("(SayText \"%s\")", word.latin1());
	if ( runMode == FESTIVAL_CMD )
		process->writeToStdin( cmd );
	else
		socket->writeBlock( cmd.latin1(), cmd.length() );
}
/*----------------------------------------------------------------------------*/
#define VD_FRAME_SHADOW			QFrame::Raised
#define VD_FRAME_STYLE			QFrame::WinPanel|QFrame::Raised
/*----------------------------------------------------------------------------*/
VDContentViewer *viewer = NULL;
/*----------------------------------------------------------------------------*/
VDContentViewer::VDContentViewer( QWidget *parent ) : QFrame( parent ), festival()
{
	QGridLayout *grid = new QGridLayout( this, 0, 0, 4 );

	word = new QLabel( this );
	contents = new VDTextBrowser( this );
	struct {
		char *tip;
		char *disable;
		const char *signal;
		QToolButton *button;
	} b[] = {
		{ "History",  0,          SIGNAL( list() ) },
		{ "Previous", 0,          SIGNAL( prev() ) },
		{ "Next",     0,          SIGNAL( next() ) },
		{ "Speech",   "NoSpeech", SLOT( sayText() ) },
		{ "New",      "NoNew",    SIGNAL( edit() ) },
		{ "Edit",     "NoEdit",   SIGNAL( edit() ) },
		{ "Delete",   "NoDelete", SIGNAL( edit() ) },
		{ 0, 0 }
	};

	for( int i = 0; b[i].tip; i++ ) {
		QToolButton *button = new QToolButton( this );
		connect( button, SIGNAL( clicked() ), b[i].signal );
		QToolTip::add( button, b[i].tip );

		if (!b[i].disable)
			button->setPixmap( vdLoadImage( b[i].tip ) );
		else {
			QIconSet icons( vdLoadImage(b[i].tip) );
			icons.setPixmap( vdLoadImage(b[i].disable),
					QIconSet::Automatic, QIconSet::Disabled );
			button->setIconSet( icons );
		}
		button->setFixedSize( 24, 24 );
		button->setAutoRaise( true );
		button->setEnabled( i < 3 );
		button->setBackgroundMode( Qt::PaletteBackground );
		b[i].button = button;
	}

	QLabel *separator = new QLabel( this );
	separator->setFrameShape( QFrame::VLine );
	separator->setFrameShadow( QFrame::Sunken );
	grid->setColStretch( 5, 1 );
	grid->addWidget( b[0].button, 0, 0 );
	grid->addWidget( b[1].button, 0, 1 );
	grid->addWidget( b[2].button, 0, 2 );
	grid->addWidget( separator, 0, 3 );
	grid->addWidget( speech = b[3].button, 0, 4 );
	grid->addWidget( word, 0, 5 );
	grid->addWidget( b[4].button, 0, 6 );
	grid->addWidget( b[5].button, 0, 7 );
	grid->addWidget( b[6].button, 0, 8 );
	grid->addMultiCellWidget( contents, 1, 1, 0, 8 );
	this->setFrameStyle( VD_FRAME_STYLE );
	this->setFrameShadow( VD_FRAME_SHADOW );
	contents->setPaper( Resource::getBgPaper() );
	word->setFixedHeight( 20 );

	connect( &festival, SIGNAL( ready() ), SLOT( enableSpeech() ) );
	connect( &festival, SIGNAL( stop() ), SLOT( disableSpeech() ) );
	if ( Resource::speechEnabled )
		festival.open();

	connect( contents, SIGNAL( getlink(const QString&) ), SLOT( getlink(const QString&) ));
	help = NULL;
}
/*----------------------------------------------------------------------------*/
void VDContentViewer::applyDefaultFont()
{
	QFont f = Resource::defaultFont;
	f.setBold( true );
	contents->setFont( Resource::defaultFont );
	word->setFont( f );
}
/*----------------------------------------------------------------------------*/
void VDContentViewer::applyDefaultColor()
{
	contents->applyDefaultColor();
}
/*----------------------------------------------------------------------------*/
void VDContentViewer::resetSpeech()
{
	festival.reset();
}
/*----------------------------------------------------------------------------*/
void VDContentViewer::enableSpeech()
{
	speech->setEnabled( true );
}
/*----------------------------------------------------------------------------*/
void VDContentViewer::disableSpeech()
{
	speech->setEnabled( false );
}
/*----------------------------------------------------------------------------*/
void VDContentViewer::sayText()
{
	festival.say( word->text() );
}
/*----------------------------------------------------------------------------*/
bool VDContentViewer::systemLink( const QString& w )
{
	if( w.startsWith( "vd://" ) ) {
		if( w == "vd://back" )
			contents->backward();
		else
		if( w == "vd://make_index" ) {
			word->setText( QString::fromUtf8( "Cách tạo index table" ) );
			contents->setText( help!=NULL ? help : "" );
		}
	}
	else
	if( w.startsWith( "http://" ) ) {
		QString cmd = QString("netscape ") + w + " &";
		system( cmd );
	}
	else
	if( w.startsWith( "mailto://" ) ) {
		// w.replace( 0, 9 );
		// w.prepend( mailClient + " " );
		// system( w );
	}
	else return false;

	return true;
}
/*----------------------------------------------------------------------------*/
void VDContentViewer::getlink( const QString& w )
{
	if( !systemLink( w ) )
		emit reference( w );
}
/*----------------------------------------------------------------------------*/
