#!/usr/bin/env python
#----------------------------------------------------------------------------
import sys, time, gdbm, getopt, readline, tempfile, struct
#----------------------------------------------------------------------------
VD_SIGNATURE = "VDFM2002";
VD_CHECKSUM = "VDFMCKSM";
VD_DESCRIPTION = "VDFMDESC";
VD_INPUTMETHOD = "VDFMIMED";
#----------------------------------------------------------------------------
VD_DBFNAME = "VDFMNAME";
VD_DBFINFO = "VDFMINFO";
VD_DBFURL = "VDFMURLS";
#----------------------------------------------------------------------------
methods = ("Latin input method", "Vietnamese input method");
#----------------------------------------------------------------------------
def usage():
	print "\n* Usage:";
	print "   ", sys.argv[0],"[options]";
	print "\t      -s <webdict.db>   Input HND's WebDict database";
	print "\t      -d <vd.vdbf>      Output Visual Dictionary database";
	print "\t      -i <vd.vdbi>      Output Visual Dictionary index file";
	print "\t      -m <vn|lat>       Using Vietnamese/Latin input method (default Latin)";
	print "\t      -x <description>  Dictionary's description (default input)\n";
	sys.exit( -1 );
	return
#----------------------------------------------------------------------------
def create_vdbf(sfn, dfn, ifn, method, description):
	date = time.time();
	ofp = open( tempfile.mktemp(), "w+r" );
	sfp = open( sfn, "r" );
	ifp = open( ifn, "w+" );

	if not sfp:
		print "Cannot open source file: %s!\n", sfn
		return 0

	if not ifp:
		print "Create index[%s] error!\n", ifn
		return 0

	dfp = gdbm.open( dfn, "n" );
	if dfp == None:
		print "Create db[%s] error: \n" % dfn;
		return 0

	idx = count = checksum = offset = 0;
	ifp.write( VD_SIGNATURE );
	ifp.write( struct.pack("l", date) );
	ifp.write( struct.pack("l", method) );
	info_offset = ifp.tell();

	ifp.write( struct.pack("l", count) );
	ifp.write( struct.pack("L", checksum) );
	ifp.write( struct.pack("l", idx) );

	q = "";
	while 1:
		p = sfp.readline();
		if p == "": break;
		l = len( p );
		if p[0] != '@':
			q += p
		else:
			qlen = len(q);
			if qlen > 0:
				if len(word) > 0:
					if word[:12] == "00-database-":
						if word[12:] == "info":
							dfp[VD_DBFINFO] = q;
						elif word[:12] == "url":
							dfp[VD_DBFURL] = q;
						elif word[:12] == "short":
							dfp[VD_DBFNAME] = q;
					else:
						dfp[word] = q;
						ofp.write(struct.pack("l", offset));
						ifp.write(word);
						ifp.write("\0");
						offset += len(word) + 1;
				q = ""

			count = count + 1;
			print "Word counts = %d\r" % count,

			w = p[1:];
			p = w.find( " /" );
			if p >= 0:
				q += w[p + 1:];
				w = w[:p];
			else:
				q = "\n";
			word = w.strip();
			for c in word: checksum += ord(c);

		if len(q) > 60000:
			sfp.close();
			ofp.close();
			ifp.close();
			dfp.close();
			print "It seems source file is not HND's WebDict database!";
			sys.exit( 0 );

	sfp.close();

	if len(q) > 0:
		dfp[word] = q;
		ofp.write( struct.pack("l", offset) );
		ifp.write( word );
		ifp.write( "\0" );
	idx = ifp.tell();

# Append all index table to index file
	ofp.seek( 0 );
	ifp.write( ofp.read() );
	ofp.close();

# Update all keys to db
	dfp[VD_SIGNATURE] = time.ctime(date) + "\n";
	dfp[VD_DESCRIPTION] = description;
	dfp[VD_INPUTMETHOD] = methods[method];
	dfp[VD_CHECKSUM] = str(checksum);
	dfp.close();

# Update all other info to index file
	ifp.seek( info_offset );
	ifp.write( struct.pack("l", count) );
	ifp.write( struct.pack("L", checksum) );
	ifp.write( struct.pack("l", idx) );
	ifp.close();
	return 1;
#----------------------------------------------------------------------------
def dup_fname(s, ext):
	p = s.rfind('/');
	if p >= 0: s = s[p+1:];
	f = s;
	p = f.rfind('.');
	if p >= 0:
		f = f[:p] + ext;
	else:
		f += ext;
	return f;
#----------------------------------------------------------------------------
def main():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "s:d:i:m:x:");
	except getopt.GetoptError:
		usage();

	method = 0;
	sfn = dfn = ifn = description = None;

	for o, a in opts:
		if o == "-s": sfn = a;
		if o == "-d": dfn = a;
		if o == "-i": ifn = a;
		if o == "-m" and a == "vn": method = 1
		if o == "-x": description = a;

	if not sfn: usage();
	if not dfn: dfn = dup_fname(sfn, ".vdbf");
	if not ifn: ifn = dup_fname(sfn, ".vdbi");
	if not description:
		description = raw_input("Description (24 chars): ");

	print "wddb = ", sfn;
	print "vdbf = ", dfn;
	print "vdbi = ", ifn;
	print "imed = ", methods[method];
	print "desc = ", description;

	create_vdbf( sfn, dfn, ifn, method, description );
	print "\ndone.\n";

#----------------------------------------------------------------------------
if __name__ == "__main__":
	main();
